#include "primitives.h"
#include "physics.h"
#include "mesh.h"

typedef struct _cloth {
  MeshRef mesh;
  
  // structure. Nodes and springs are kept in an array, but are still linked in a list
  PointMassRef nodeBuffer;
  SpringRef springBuffer; 
  long springCount;
  
  // number of nodes horizontaly and vertically
  long width;
  long height;
  
  // parameters
  double dampingFactor;
  Vector gravity;
  
  // external constaints
  SphereRef spheres;
  PlaneRef planes;
  NailRef nails;
} Cloth;

typedef Cloth * ClothRef;

// creates point masses and springs for all vertices
ClothRef ClothMake(MeshRef mesh, Vector gravity, double damp, double stretch, double shear, double bend); 

// adding external constraints
void ClothAddSphere(ClothRef self, SphereRef newSphere);
void ClothAddPlane(ClothRef self, PlaneRef newPlane);
void ClothAddNail(ClothRef self, long x, long y);

void ClothStep(ClothRef self, double dt);
