#include "drawing.h"
#include <stdlib.h>
#include <GL/glut.h>

SceneRef MainScene() {
  static SceneRef scene = NULL;
  
  if(scene != NULL)
    return scene;
    
  scene = SceneMake();
  
  return scene;
}

SceneRef SceneMake() {
  SceneRef scene; 
  
  scene = (SceneRef) malloc(sizeof(Scene));
  scene->width = 640;
  scene->height = 480;
  scene->mesh = NULL;
  
  scene->cameraPosition = PointMakeZero();
  scene->cameraTarget =  PointMakeZero();
  scene->lightDirection = VectorMakeZero();
  
  const GLfloat light0Ambient[] = {0.001, 0.001, 0.001, 1.0};
  const GLfloat light0Diffuse[] = {1.0, 1.0, 1.0, 1.0};
  const GLfloat light0Specular[] = {1.0, 1.0, 1.0, 1.0};
  const GLfloat matAmbient[] = {0.6, 0.6, 0.6, 1.0};
  const GLfloat matDiffuse[] = {1.0, 1.0, 1.0, 1.0};
  
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, matAmbient);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, matDiffuse);
      
  glLightfv(GL_LIGHT0, GL_AMBIENT, light0Ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0Diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light0Specular);
  
  glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, 1.0);
  
  glEnable(GL_NORMALIZE);  
  glEnable(GL_DEPTH_TEST);

//  glEnable(GL_POLYGON_SMOOTH);
  
  Point cp = PointMake(0.0, 0.0, 0.0), ct = PointMake(0.0, 0.0, 0.0);
  SceneSetLight(scene, VectorMake(0.0, 0.0, 0.0));
  SceneSetCamera(scene, cp, ct);
  
  return scene;
}

void SceneSetMesh(SceneRef self, MeshRef mesh) {
  self->mesh = mesh;
}

void SceneSetLight(SceneRef self, Vector direction) {
  // VectorFree(self->lightDirection);
  self->lightDirection = direction;
  
  GLfloat light[4] = {self->lightDirection.u, self->lightDirection.v, self->lightDirection.w, 0};
  glLightfv(GL_LIGHT0, GL_POSITION, light);
}

void SceneSetCamera(SceneRef self, Point cp, Point ct) {
  self->cameraPosition = cp;
  self->cameraTarget = ct;
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  
  gluLookAt(cp.x, cp.y, cp.z, ct.x, ct.y, ct.z, 0.0, 1.0, 0.0);
  
  GLfloat light[4] = {self->lightDirection.u, self->lightDirection.v, self->lightDirection.w, 0};
  glLightfv(GL_LIGHT0, GL_POSITION, light);
}

void SceneRotateCamera(SceneRef self, Vector axis, GLfloat angle) {
  Point ct = self->cameraTarget;
  
  glMatrixMode(GL_MODELVIEW);
  
  glTranslatef(ct.x, ct.y, ct.z);
  glRotatef(angle, axis.u, axis.v, axis.w);
  glTranslatef(-ct.x, -ct.y, -ct.z);
  
  GLfloat light[4] = {self->lightDirection.u, self->lightDirection.v, self->lightDirection.w, 0};
  glLightfv(GL_LIGHT0, GL_POSITION, light);
}

void SceneUpdateViewport(SceneRef self, int width, int height) {
  self->width = width;
  self->height = height;
  
  float ratio = 1.0 * width / (height ? height : 1.0);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glViewport(0, 0, width, height);
  gluPerspective(45,ratio,1,1000);
}

void SceneRender(SceneRef self) {
  MeshDraw(self->mesh);
  
  glFlush();
}

