#include <stdlib.h>
#include <GL/glut.h>
#include "drawing.h"
#include "cloth.h"
#include "input.h"

ClothRef theCloth;

void display(void) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  SceneRender(MainScene());
  glutSwapBuffers();
}

void reshape(int width, int height) {
  SceneUpdateViewport(MainScene(), width, height);
}

void idle(void) {
  
}

void timestep(int value) {
  static int msec = 0;

  ClothStep(theCloth, 0.025);
  if(msec > 25) {
    glutPostRedisplay();
    msec = 0;
  }
  
  glutTimerFunc(15, timestep, 0);
  msec += 15;
}

void init(void) {
  SceneRef scene = MainScene();
  SceneSetLight(scene, VectorMake(-1.0, -1.0, -1.0));
  SceneSetCamera(scene, PointMake(8.0, 8.0, 8.0), PointMake(0.0, 3.0, 0.0));
  
  MeshRef mesh = MeshCreateGrid(25, 25, 5.0, 5.0, 5.0);
  SceneSetMesh(scene, mesh);
  
  // prototype: ClothMake(mesh, gravity, momentum damping, stretch stiffnes, shear stiffness, bend stiffness);
  theCloth = ClothMake(mesh, VectorMake(0.0, -1.0, 0.0), 0.02, 0.9, 0.7, 0.1);
  
  // add nail constraint
  // ClothAddNail(theCloth, 0, 0);
  // ClothAddNail(theCloth, 24, 0);
  
  // add a sphere constrain, just for effect
  SphereRef sphere1 = (SphereRef) malloc(sizeof(Sphere));
  SphereRef sphere2 = (SphereRef) malloc(sizeof(Sphere));
  *sphere1 = SphereMake(PointMake(-0.8, 3.7, 0.0), 1.0, 0.2);
  *sphere2 = SphereMake(PointMake(1.0, 1.0, 0.5), 1.5, 0.1);
  
  ClothAddSphere(theCloth, sphere1);
  ClothAddSphere(theCloth, sphere2);
  
  PlaneRef plane = (PlaneRef) malloc(sizeof(Plane));
  *plane = PlaneMake(VectorMake(0.0, 1.0, 0.0), 1.0);
  
  ClothAddPlane(theCloth, plane);
}

int main(int argc, char** argv) {
    glutInit(&argc, argv);
    
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize(640, 480);
    
    glutCreateWindow("Cloth simulation demo");
    
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
    glutIdleFunc(idle);
    
    glutMouseFunc(MouseButton);
    glutMotionFunc(MouseDragged);
    
    init();
    
    glutTimerFunc(100, timestep, 0);
    glutMainLoop();
    return EXIT_SUCCESS;
}

