#ifndef PHYSICS_H
#define PHYSICS_H

#include <GL/glut.h>
#include "primitives.h"

typedef struct _pointmass {
  VertexRef vertex; // includes current position
  GLfloat lastPosition[3]; // could go with Point, but preferred to have the same data type as the vertex...
  Vector force;
} PointMass;

typedef PointMass * PointMassRef;

PointMass PointMassMake(VertexRef vertex);
void PointMassFree();

void PointMassIntegrate(PointMassRef self, double damp, double dt);

typedef struct _spring {
  PointMassRef a;
  PointMassRef b;
  double length;
  double stiffness;
} Spring;

typedef Spring * SpringRef;

Spring SpringMake(PointMassRef a, PointMassRef b, double stiffness);

void SpringSatisfy(SpringRef self);

typedef struct _sphere {
  Point center;
  double radius;
  double friction;
  struct _sphere *next;
} Sphere;

typedef Sphere * SphereRef;

// Note that even friction set to 1.0 isn't going to stop the cloth from sliding off the sphere eventually
Sphere SphereMake(Point center, double radius, double friction);
void SphereSatisfyForPointMass(SphereRef self, PointMassRef pointMass);

typedef struct _plane {
  Vector normal;
  double shift;
  struct _plane *next;
} Plane;

typedef Plane * PlaneRef;

Plane PlaneMake(Vector normal, double shift);
void PlaneSatisfyForPointMass(PlaneRef self, PointMassRef pointMass);

typedef struct _nail {
  PointMassRef node;
  Point origin;
  struct _nail *next; // next in the list
} Nail;

typedef Nail * NailRef;

Nail NailMake(PointMassRef node);
void NailSatisfy(NailRef self);

#endif PHYSICS_H
