#ifndef PRIMITIVES_H
#define PRIMITIVES_H

#include <GL/glut.h>

// Basic math primitives

typedef struct _point {
  GLfloat x;
  GLfloat y;
  GLfloat z;
} Point; 

Point PointMake(GLfloat x, GLfloat y, GLfloat z);
Point PointMakeZero();

Point PointAddPoint(Point v1, Point v2);
Point PointSubPoint(Point v1, Point v2);

typedef struct _vector {
  GLfloat u;
  GLfloat v;
  GLfloat w;
} Vector;

Vector VectorMake(GLfloat u, GLfloat v, GLfloat w);
Vector VectorMakeZero();

double VectorLength(Vector v);
Vector CrossProduct(Vector const v1, Vector const v2);

// Basic geometry primitives

typedef struct _vertex {
  // stored as arrays because of the way OpenGL handles vertex and normal arrays
  GLfloat position[3];
  GLfloat normal[3];
} Vertex;

typedef Vertex * VertexRef;

Vertex VertexMake(Point position, Vector normal);

#endif